@@ -195,6 +195,16 @@ luaH_class_emit_signal(lua_State *L, lua_class_t *lua_class,
     return signal_object_emit(L, lua_class->signals, name, nargs, nret);
 }
 
+gint
+luaH_class_property_signal(lua_State *L, lua_class_t *lua_class,
+        luakit_token_t tok)
+{
+    gchar *signame = g_strdup_printf("property::%s", token_tostring(tok));
+    signal_object_emit(L, lua_class->signals, signame, 0, 0);
+    g_free(signame);
+    return 0;
+}
+
 /* Try to use the metatable of an object.
  * `idxobj` is the index of the object.
  * `idxfield` is the index of the field (attribute) to get.
