@@ -112,10 +112,10 @@ window.follow = (function () {
         };
         // sort top to bottom and left to right
         elements.sort(function (a,b) {
-            return a.getBoundingClientRect().left - b.getBoundingClientRect().left;
+            return a.getClientRects()[0].left - b.getClientRects()[0].left;
         });
         elements.sort(function (a,b) {
-            return a.getBoundingClientRect().top - b.getBoundingClientRect().top;
+            return a.getClientRects()[0].top - b.getClientRects()[0].top;
         });
         return elements;
     }
@@ -134,6 +134,7 @@ window.follow = (function () {
     function Hint(element) {
         this.element = element;
         this.rect = element.getBoundingClientRect();
+        this.rect0 = element.getClientRects()[0];
 
         // Hint creation helper functions.
         function createSpan(hint, h, v) {
@@ -143,8 +144,8 @@ window.follow = (function () {
                 leftpos = document.defaultView.scrollX;
                 toppos = document.defaultView.scrollY;
             } else {
-                leftpos = Math.max((hint.rect.left + document.defaultView.scrollX), document.defaultView.scrollX) + h;
-                toppos = Math.max((hint.rect.top + document.defaultView.scrollY), document.defaultView.scrollY) + v;
+                leftpos = Math.max((hint.rect0.left + document.defaultView.scrollX), document.defaultView.scrollX) + h;
+                toppos  = Math.max((hint.rect0.top  + document.defaultView.scrollY), document.defaultView.scrollY) + v;
             }
             // ensure all hints are visible
             leftpos = Math.max(leftpos, 0);
@@ -156,7 +157,7 @@ window.follow = (function () {
         }
 
         function createTick(hint) {
-            var tick = createSpan(hint, follow.theme.horiz_offset, follow.theme.vert_offset - hint.rect.height/2);
+            var tick = createSpan(hint, follow.theme.horiz_offset, follow.theme.vert_offset - hint.rect0.height/2);
             tick.style.font = follow.theme.tick_font;
             tick.style.color = follow.theme.tick_fg;
             if (isFrame(hint.element)) {
@@ -174,8 +175,8 @@ window.follow = (function () {
 
         function createOverlay(hint) {
             var overlay = createSpan(hint, 0, 0);
-            overlay.style.width = hint.rect.width + "px";
-            overlay.style.height = hint.rect.height + "px";
+            overlay.style.width  = (hint.rect.left + hint.rect.width   - hint.rect0.left) + "px";
+            overlay.style.height = (hint.rect0.top + hint.rect0.height - hint.rect.top)   + "px";
             overlay.style.opacity = follow.theme.opacity;
             overlay.style.border = follow.theme.border;
             overlay.style.zIndex = 10000;
@@ -488,7 +489,7 @@ end
 
 -- Check if following is possible safely
 local function is_ready(w)
-    for _, f in ipairs(w:get_current().frames) do
+    for _, f in ipairs(w.view.frames) do
         local ret = w:eval_js("!!(document.body && /interactive|loaded|complete/.test(document.readyState) && window.follow)", "(follow.lua)", f)
         if ret ~= "true" then return false end
     end
@@ -502,7 +503,7 @@ local function focus(w, offset)
         return (w:eval_js("follow.focused();", "(follow.lua)", f) == "true")
     end
     -- sort frames with currently active one first
-    local frames = w:get_current().frames
+    local frames = w.view.frames
     if #frames == 0 then return end
     for i=1,#frames,1 do
         if is_focused(frames[1]) then break end
@@ -563,7 +564,7 @@ local function accept_follow(w, frame)
             val = string.match(ret, "done (.*)")
         end
     else
-        for _, f in ipairs(w:get_current().frames) do
+        for _, f in ipairs(w.view.frames) do
             local ret = w:eval_js("follow.evaluate();", "(follow.lua)", f)
             local done = string.match(ret, "^done")
             if done then
@@ -622,7 +623,7 @@ new_mode("follow", {
         local theme_js = mk_theme_js()
 
         local sum = 0
-        local webkit_frames = w:get_current().frames
+        local webkit_frames = w.view.frames
         local ignore_frames = tostring(#webkit_frames == 1)
         local m = state.mode
 
@@ -674,7 +675,7 @@ new_mode("follow", {
     -- Leave follow mode hook
     leave = function (w)
         if w.eval_js then
-            for _,f in ipairs(w:get_current().frames) do
+            for _,f in ipairs(w.view.frames) do
                 w:eval_js(clear_js, "(follow.lua)", f)
             end
         end
@@ -689,7 +690,7 @@ new_mode("follow", {
         local eval_frame
         local split = lousy.util.string.split
         local filter_js = string.format("follow.filter(%q, %q);", filter, id)
-        for _, f in ipairs(w:get_current().frames) do
+        for _, f in ipairs(w.view.frames) do
             local ret = split(w:eval_js(filter_js, "(follow.lua)", f))
             if ret[2] == "true" then focus(w, 1) end -- Reselect active hint
             local num = tonumber(ret[1])
